/*  GnuPOS-P.h                         */
/*  header file for private data       */

typedef struct
{
  GdkColor   *fg;  /* Foreground colour */
  GdkColor   *bg;  /* Background colour */
  GdkFont    *font; /* Font to use      */
  gint       height; /* Size (in pixels) */

  gchar       textbuf[250];
                   /* Text buffer       */
} topbox_data_struct;

typedef struct
{
  GdkColor   *fg;  /* Foreground colour */
  GdkColor   *bg;  /* Background colour */
  GdkFont    *font; /* Font to use      */

  gchar       textbuf[250];
                   /* Text buffer       */
} rightbox_data_struct;

typedef struct
{
  GdkColor   *fg;  /* Foreground colour */
  GdkColor   *bg;  /* Background colour */
  GdkFont    *font; /* Font to use      */

  gchar       textbuf[250];
                   /* Text buffer       */
} leftbox_data_struct;

typedef struct
{
  GtkWidget  *topbox;          /* pointer to it  */
  GtkWidget  *rightbox;        /* pointer to it  */
  GtkWidget  *leftbox;         /* pointer to it  */

  topbox_data_struct   top;    /* topbox   */
  rightbox_data_struct right;  /* rightbox */
  leftbox_data_struct  left;   /* leftbox  */
  
  guint                 height; /* Height of rightbox and leftbox */
  guint                 width;  /* Width of topbox (total window width) */
  gboolean             show_right; /* rightbox visible? */
  gboolean             show_left;  /* leftbox visible?  */
} mainwindow_data_struct;

typedef struct
{
  guint           function;     /* funciton to execute        */
  guint           param_count;   /* # of parameters to pass    */
  guint           param1;       /* These are the register     */
  guint           param2;       /* codes for the register to  */
  guint           param3;       /* be used for this parameter */
  guint           param4;
  void           *next;         /* pointer to next instance   */
} commands_struct;

typedef struct
{
  guint            inpchar;          /* char/special that triggers input */
  guint            endstate;         /* State to switch to if match      */
  commands_struct *command;          /* function to perform              */
} state_input_struct;

typedef struct
{
  state_input_struct *inputs;      /* pointer to array of inputs       */
  gchar              *helptext;    /* pointer to helptext to display   */
  guint               numinputs;   /* counter of how many inputs exist */
} state_instance_struct;

typedef struct
{
  state_instance_struct *states;       /* array of all states      */
  gchar                 *(*nametable); /* array of state names     */
  guint                 max_states;    /* count of how many states */
  guint                 current_state; /* current state index      */
} state_master_struct;

typedef struct
{
/* There is a hack here. We use sub to contain a pointer to a function */
/* but have to rely on code > 0xfeffffff to flag that we should use it.*/
  gchar              *sym;          /* name of function  */
  guint               code;         /* function code     */
  guint               param_count;  /* number of paramters required */
  commands_struct    *command;      /* function pointer  */
  void               (*sub)();      /* pointer to subroutine */
  guint               endstate;     /* endstate          */
} functions_struct;

typedef struct
{
  gchar              *sym;        /* Register name  */
  guint               code;       /* register code  */
  guint               type;       /* type           */
  guint               value_type; /* Type of current value */
  guint               value;
} registers_struct;
