/*  GnuPOS-keyboard-P.h    */
/*  Private stuff for GnuPOSkeyboard.c */

/* The keysym table */
typedef struct
{
  char      *sym;      /* the symbol that represents the above value */
  guint     code;      /* the value attached to this symbol */
} symval_struct;

/* prototypes */
void             gnp_read_keysyms(void);
void             gnp_read_keymaps(void);
void             gnp_initialise_functions(void);
void             gnp_initialise_registers(void);
gboolean         gnp_parse_state(xmlDocPtr,xmlNodePtr,guint);
guint            gnp_lookup_key(gchar *);
gchar           *gnp_lookup_key_name(guint);
guint            gnp_lookup_state(gchar *);
gchar           *gnp_lookup_state_name(guint);
guint            gnp_lookup_function_code_by_name(gchar *);
gchar           *gnp_lookup_function_name(guint);
guint            gnp_lookup_parameter(gchar *);
gchar           *gnp_lookup_parameter_name(guint);
commands_struct *gnp_parse_command(gchar *);
guint            gnp_scan_eos(gchar *);

/* Prototypes for processing  */
void             gnp_execute_input(state_input_struct *);
void             gnp_perform_command(commands_struct *);
void             gnp_set_register_value(guint,guint,guint);
void             gnp_set_register_value_privileged(guint,guint,guint);

/*  State Program calls  */
void             gnp_reset_func(void);
void             gnp_clear_func(guint);

/* Inbuilt functions */
/* Do NOT count UNDEF when determining the count! */
#define GNP_INTERNAL_FUNCTION_COUNT 11

#define GNPF_UNDEF     0xffffffff
#define GNPF_RESET     0xff000000
#define GNPF_CLEAR     0xff000001
#define GNPF_ADD       0xff000002
#define GNPF_SUBTRACT  0xff000003
#define GNPF_MULTIPLY  0xff000004
#define GNPF_DIVIDE    0xff000005
#define GNPF_REMAINDER 0xff000006
#define GNPF_MOVE      0xff000007
#define GNPF_DISPLAY   0xff000008
#define GNPF_OUTPUT    0xff000009
#define GNPF_QUIT      0xff00000a

/* Inbuilt registers */
/* Do NOT count UNDEF when determining the count! */
#define GNP_INTERNAL_REGISTER_COUNT 9
#define GNPR_UNDEF      0xffffffff
#define GNPR_KEY        0xff000000  /* register that contains keypress value */
#define GNPR_RETURN     0xff000001  /* register that contains return value */
#define GNPR_STATENAME  0xff000002  /* register that contains the name of the current state */
#define GNPR_STRING     0xff200000  /* register for passing pointers to string constants */
#define GNPR_NUMBER     0xff200001  /* register for passing integer constants            */
#define GNPR_MODETEXT   0xff100000  /* Modetext display register */
#define GNPR_DISPLAY    0xff100001  /* Topbox display register */
#define GNPR_HELPTEXT   0xff100002  /* Rightbox display register */
#define GNPR_LOGTEXT    0xff100003  /* Leftbox display register */
#define GNPR_STATUSTEXT 0xff100004  /* Status bar display register */


/* Inbuilt States  */
/* Do NOT count UNDEF when determining the count! */
#define GNP_INTERNAL_STATE_COUNT 0

#define GNPS_UNDEF     0xffffffff

/* Register Types     */
#define GNP_REGISTER_DISPLAY 0x0001  /* Special display register - Type == STRING*/
#define GNP_REGISTER_SPECIAL 0x0002  /* Special purpose register - CANNOT BE CHANGED by state progs! */
#define GNP_REGISTER_PRIVATE 0x0003  /* Register for internal use only  */
#define GNP_REGISTER_NORMAL  0x0004  /* Normal register for state program use */
#define GNP_REGISTER_FIXED   0x0005  /* Fixed type register for state program use */

/*  Register value Types  */
#define GNP_VALUE_UNDEF      0xffff
#define GNP_VALUE_STRING     0x0001  /* Value is pointer-to-string */
#define GNP_VALUE_INTEGER    0x0002  /* Value is integer           */
#define GNP_VALUE_GNCNUMERIC 0x0003  /* Value is pointer-to-gnc_numeric */

/*  Command structure types  */
#define GNP_COMMAND_INTERNAL 0x0001  /* Command is an internal command */
#define GNP_COMMAND_DEFINED  0x0002  /* Command is a defined function  */
#define GNP_COMMAND_EXTERNAL 0x0003  /* Command is an external command (for future use) */

/*  Predefined special keys  */
#define GNP_INTERNAL_KEY_COUNT 10
#define GNPK_NUMERIC         0xff000000
#define GNPK_LOWERCASE       0xff000001
#define GNPK_UPPERCASE       0xff000002
#define GNPK_ALPHA           0xff000003
#define GNPK_ALPHNUM         0xff000004
#define GNPK_PUNCT           0xff000005
#define GNPK_SPACES          0xff000006
#define GNPK_PRINTING        0xff000007
#define GNPK_CONTROL         0xff000008
#define GNPK_DEFAULT         0xffffffff

