/*  GnuPOSRealStuff.c                  */
/*  This contains the interface routines  */
/*  for GnuPOS. This file should contain: */
/*    - Anything referenced by more than  */
/*      one callback.                     */
/*    - Support routines, etc. for above. */
/*                                        */
/*  (c) 2001 Conrad Canterford.           */

#include <gnome.h>
#include <gdk/gdk.h>

#include "support.h"
#include "GnuPOS.h"
#include "GnuPOS-P.h"

mainwindow_data_struct gnp_mainwindow_data;

extern gnp_global_data_struct gnp_global_data;

void gnp_do_exit ()

/*   Do any processing require before we  */
/*  close the application, then do the    */
/*  close (through gtk_main_quit()).      */

{
  gtk_main_quit();
}

void gnp_setup_property_dialog()

/*  Setup and display the property dialog  */

{
  if(gnp_global_data.property_dialog == NULL)
  {
    /*  create property dialog  */
    gnp_global_data.property_dialog = create_propertybox1 ();
    
    /* should feed in current values at this point */
  }
  gtk_widget_show (gnp_global_data.property_dialog);

}
/*                                         */
/*           TOPBOX ROUTINES               */
/*                                         */
/*  Grouping together all utility routines */
/* for dealing with the topbox.            */

void gnp_initialise_topbox()

/*  Setup intial values for topbox         */
/*  Eventually this must include reading   */
/*  the preferences for the different bits */
/*  from whereever they're kept and        */
/*  applying them.                         */

{
  gnp_mainwindow_data.topbox = lookup_widget(gnp_global_data.GnuPOS,"Current_entry");
  
  /*  Load values for things here  */
  /*  these lines should be replaced when proper prefs loading */
  /*  is happening here instead. */
  gnp_mainwindow_data.top.height = 40;
  gnp_mainwindow_data.top.font = NULL;
  
  gnp_topbox_resize();
  gnp_topbox_refont();
  gtk_text_set_word_wrap(gnp_mainwindow_data.topbox, TRUE);
}

void gnp_topbox_resize()

/*  Adjust the size of topbox    */

{
  GtkWidget *heightwidget;
  GtkWidget *widthwidget;
  GtkWidget *scroller;
  guint      oldwidth;

  oldwidth=gnp_mainwindow_data.width;
  
  if(gnp_global_data.property_dialog != NULL)
  {
    heightwidget = lookup_widget(gnp_global_data.property_dialog, "topboxspin");
    widthwidget = lookup_widget(gnp_global_data.property_dialog, "topboxwidthspin");

    gnp_mainwindow_data.top.height = gtk_spin_button_get_value_as_int(heightwidget);
    gnp_mainwindow_data.width = gtk_spin_button_get_value_as_int(widthwidget);
  }

  scroller = lookup_widget(gnp_global_data.GnuPOS, "topboxscroller");

  if(oldwidth!=gnp_mainwindow_data.width)
  {
    gnp_autohist_resize();
  }

  gtk_widget_set_usize(scroller, gnp_mainwindow_data.width, gnp_mainwindow_data.top.height);

}

void gnp_topbox_refont()

/*  Make necessary modifications for a     */
/*  change of font for topbox.             */
/*  Includes decreasing the font count for */
/*  the old font, and increasing the font  */
/*  count for the new font.                */

{
  GtkWidget *fontwidget;
  
  if(gnp_global_data.property_dialog != NULL)
  {
    fontwidget = lookup_widget(gnp_global_data.property_dialog, "topboxfont");

    gdk_font_unref(gnp_mainwindow_data.top.font);
    gnp_mainwindow_data.top.font = gnome_font_picker_get_font(fontwidget);
    gdk_font_ref(gnp_mainwindow_data.top.font);
  }
}

void gnp_topbox_erase_all()

/* Completely erase the entire topbox      */
/* At this stage, does not check to ensure */
/*  that it worked, and returns nothing.   */

{
  guint      length;
  
  gtk_text_freeze(gnp_mainwindow_data.topbox);
  length = gtk_text_get_length(gnp_mainwindow_data.topbox);
  gtk_text_set_point(gnp_mainwindow_data.topbox,0);
  gtk_text_forward_delete(gnp_mainwindow_data.topbox,length);
  gtk_text_thaw(gnp_mainwindow_data.topbox);
}

void gnp_topbox_insert_char(gchar x)

/*  Insert the character x to the right of */
/* the last character already present.     */
/* Just wraps gtk_text_insert()            */

{
  gchar      buffer[2];
  
  sprintf(buffer,"%c",x);
  buffer[1]='\0';
  
  gtk_text_insert(gnp_mainwindow_data.topbox, gnp_mainwindow_data.top.font,
                  gnp_mainwindow_data.top.fg, gnp_mainwindow_data.top.bg,
                  buffer, -1);
}

void gnp_topbox_insert_string(gchar *x)

/*  Insert the string x to the right of    */
/* the last character already present.     */
/* Just wraps gtk_text_insert()            */

{
  gtk_text_insert(gnp_mainwindow_data.topbox, gnp_mainwindow_data.top.font,
                  gnp_mainwindow_data.top.fg, gnp_mainwindow_data.top.bg,
                  x, -1);
}

/*                                         */
/*           RIGHTBOX ROUTINES             */
/*                                         */
/*  Grouping together all utility routines */
/* for dealing with the rightbox.          */

void gnp_initialise_rightbox()

/*  Setup intial values for rightbox       */
/*  Eventually this must include reading   */
/*  the preferences for the different bits */
/*  from whereever they're kept and        */
/*  applying them.                         */

{
  GtkWidget *pane;
  
  gnp_mainwindow_data.rightbox = lookup_widget(gnp_global_data.GnuPOS,"rightbox");

  gnp_mainwindow_data.right.font = NULL;
  
  gnp_autohist_resize();
  gnp_rightbox_refont();
  gtk_text_set_word_wrap(gnp_mainwindow_data.rightbox, TRUE);
  pane = lookup_widget(gnp_global_data.GnuPOS, "mainpane");
  gtk_paned_set_position(pane,gnp_mainwindow_data.width/2);

}

void gnp_rightbox_hide()

/*  Unshow the rightbox  */

{
  GtkWidget *scroller;
  GtkWidget *toggle;
  GtkWidget *pane;

  toggle = lookup_widget(gnp_global_data.property_dialog, "autoprompt_hide_checkbox");
  scroller = lookup_widget(gnp_global_data.GnuPOS, "rightboxscroller");
  pane = lookup_widget(gnp_global_data.GnuPOS, "mainpane");
  
  if(gtk_toggle_button_get_active(toggle))
  {
    gtk_widget_hide(scroller);
    gtk_paned_set_position(pane,gnp_mainwindow_data.width);
  }
  else
  {
    guint rightbox_width;
    
    gtk_widget_show(scroller);
    rightbox_width = gnp_mainwindow_data.width/2;
    gtk_paned_set_position(pane,gnp_mainwindow_data.width - rightbox_width);
  }
}

void gnp_autohist_resize()

/*  Adjust the vertical size of rightbox    */

{
  GtkWidget *heightwidget;
  GtkWidget *pane;

  if(gnp_global_data.property_dialog != NULL)
  {
    heightwidget = lookup_widget(gnp_global_data.property_dialog, "autohist_heightspin");
    
    gnp_mainwindow_data.height = gtk_spin_button_get_value_as_int(heightwidget);
  }

  pane = lookup_widget(gnp_global_data.GnuPOS, "mainpane");

  gtk_widget_set_usize(pane, gnp_mainwindow_data.width,gnp_mainwindow_data.height);
}

void gnp_rightbox_refont()

/*  Make necessary modifications for a     */
/*  change of font for rightbox.             */
/*  Includes decreasing the font count for */
/*  the old font, and increasing the font  */
/*  count for the new font.                */

{
  GtkWidget *fontwidget;
  
  if(gnp_global_data.property_dialog != NULL)
  {
    fontwidget = lookup_widget(gnp_global_data.property_dialog, "rightboxfont");

    gdk_font_unref(gnp_mainwindow_data.right.font);
    gnp_mainwindow_data.right.font = gnome_font_picker_get_font(fontwidget);
    gdk_font_ref(gnp_mainwindow_data.right.font);
  }
}
void gnp_rightbox_erase_all()

/* Completely erase the entire rightbox    */
/* At this stage, does not check to ensure */
/*  that it worked, and returns nothing.   */

{
  guint      length;
  
  gtk_text_freeze(gnp_mainwindow_data.rightbox);
  length = gtk_text_get_length(gnp_mainwindow_data.rightbox);
  gtk_text_set_point(gnp_mainwindow_data.rightbox,0);
  gtk_text_forward_delete(gnp_mainwindow_data.rightbox,length);
  gtk_text_thaw(gnp_mainwindow_data.rightbox);
}

void gnp_rightbox_insert_char(gchar x)

/*  Insert the character x to the right of */
/* the last character already present.     */
/* Just wraps gtk_text_insert()            */

{
  gchar      buffer[2];
  
  sprintf(buffer,"%c",x);
  buffer[1]='\0';
  
  gtk_text_insert(gnp_mainwindow_data.rightbox, gnp_mainwindow_data.right.font,
                  gnp_mainwindow_data.right.fg, gnp_mainwindow_data.right.bg, buffer, -1);
}

void gnp_rightbox_insert_string(gchar *x)

/*  Insert the string x to the right of    */
/* the last character already present.     */
/* Just wraps gtk_text_insert()            */

{
  gtk_text_insert(gnp_mainwindow_data.rightbox, gnp_mainwindow_data.right.font,
                  gnp_mainwindow_data.right.fg, gnp_mainwindow_data.right.bg, x, -1);
}

/*                                         */
/*           leftbox ROUTINES              */
/*                                         */
/*  Grouping together all utility routines */
/* for dealing with the leftbox.           */

void gnp_initialise_leftbox()

/*  Setup intial values for leftbox        */
/*  Eventually this must include reading   */
/*  the preferences for the different bits */
/*  from whereever they're kept and        */
/*  applying them.                         */

{
  gnp_mainwindow_data.leftbox = lookup_widget(gnp_global_data.GnuPOS,"leftbox");

  gnp_mainwindow_data.left.font = NULL;
  
  gnp_autohist_resize();
  gnp_leftbox_refont();
}

void gnp_leftbox_refont()

/*  Make necessary modifications for a     */
/*  change of font for leftbox.             */
/*  Includes decreasing the font count for */
/*  the old font, and increasing the font  */
/*  count for the new font.                */

{
  GtkWidget *fontwidget;
  
  if(gnp_global_data.property_dialog != NULL)
  {
    fontwidget = lookup_widget(gnp_global_data.property_dialog, "leftboxfont");

    gdk_font_unref(gnp_mainwindow_data.left.font);
    gnp_mainwindow_data.left.font = gnome_font_picker_get_font(fontwidget);
    gdk_font_ref(gnp_mainwindow_data.left.font);
  }
}
void gnp_leftbox_erase_all()

/* Completely erase the entire leftbox    */
/* At this stage, does not check to ensure */
/*  that it worked, and returns nothing.   */

{
  guint      length;
  
  gtk_text_freeze(gnp_mainwindow_data.leftbox);
  length = gtk_text_get_length(gnp_mainwindow_data.leftbox);
  gtk_text_set_point(gnp_mainwindow_data.leftbox,0);
  gtk_text_forward_delete(gnp_mainwindow_data.leftbox,length);
  gtk_text_thaw(gnp_mainwindow_data.leftbox);
}

void gnp_leftbox_insert_char(gchar x)

/*  Insert the character x to the left of */
/* the last character already present.     */
/* Just wraps gtk_text_insert()            */

{
  gchar      buffer[2];
  
  sprintf(buffer,"%c",x);
  buffer[1]='\0';
  
  gtk_text_insert(gnp_mainwindow_data.leftbox, gnp_mainwindow_data.left.font,
                  gnp_mainwindow_data.left.fg, gnp_mainwindow_data.left.bg, buffer, -1);
}

void gnp_leftbox_insert_string(gchar *x)

/*  Insert the string x to the left of    */
/* the last character already present.     */
/* Just wraps gtk_text_insert()            */

{
  gtk_text_insert(gnp_mainwindow_data.leftbox, gnp_mainwindow_data.left.font,
                  gnp_mainwindow_data.left.fg, gnp_mainwindow_data.left.bg, x, -1);
}
