/*  GnuPOSkeyboard.c                      */
/*  This contains the keyboard interface  */
/*  routines for GnuPOS, including the    */
/*  reading and writing of the xml state  */
/*  definitions file.                     */
/*                                        */
/*  (c) 2001 Conrad Canterford.           */

#include <gnome.h>
#include <gnome-xml/xmlmemory.h>
#include <gnome-xml/parser.h>
#include <ctype.h>

#include "GnuPOS.h"
#include "GnuPOS-P.h"
#include "GnuPOS-keyboard-P.h"

extern state_master_struct all_states;

/* Table holding symbol-value pairs for */
/* functions                            */
extern functions_struct    *function_table;
extern guint                function_table_count;

/* Table holding symbol-value pairs for */
/* parameters (registers)               */
extern registers_struct    *register_table;
extern guint                register_table_count;

extern gnp_global_data_struct gnp_global_data;

    
void gnp_process_keystroke(guint keyval)

/*   Take the keystroke returned by the    */
/*  callback, and perform all necessary    */
/*  processing to display stuff, etc.      */

{
  guint i;
  gboolean processed;

  /* If --display-keyvals option given,  */
  /* display the passed keyval.          */
  
  if(gnp_global_data.display_keyvals)
    printf("%x\n",keyval);

  /* This needs to be done in strict order. */
  /* We don't want to match a more general  */
  /* code when a more specific code will    */
  /* also match.                            */

  i=0;
  processed=FALSE;
  while(i<all_states.states[all_states.current_state].numinputs&&!processed)
  {
    if(keyval==all_states.states[all_states.current_state].inputs[i].inpchar)
    {
      if(gnp_global_data.program_debug)
        printf("PROGRAM: Matched input for \'%s\'. Executing those commands.\n",gnp_lookup_key_name(keyval));
      gnp_execute_input(&all_states.states[all_states.current_state].inputs[i]);
      processed=TRUE;
    }
    i++;
  }

  if(!processed)
  {
    if(isdigit(keyval))
    {
      i=0;
      while(i<all_states.states[all_states.current_state].numinputs&&!processed)
      {
        if(all_states.states[all_states.current_state].inputs[i].inpchar==GNPK_NUMERIC)
        {
          if(gnp_global_data.program_debug)
            printf("PROGRAM: Matched input for %%numeric. Executing those commands.\n");
          gnp_execute_input(&all_states.states[all_states.current_state].inputs[i]);
          processed=TRUE;
        }
        i++;
      }
    }
  }

  if(!processed)
  {
    if(islower(keyval))
    {
      i=0;
      while(i<all_states.states[all_states.current_state].numinputs&&!processed)
      {
        if(all_states.states[all_states.current_state].inputs[i].inpchar==GNPK_LOWERCASE)
        {
          if(gnp_global_data.program_debug)
            printf("PROGRAM: Matched input for %%lowercase. Executing those commands.\n");
          gnp_execute_input(&all_states.states[all_states.current_state].inputs[i]);
          processed=TRUE;
        }
        i++;
      }
    }
  }

  if(!processed)
  {
    if(isupper(keyval))
    {
      i=0;
      while(i<all_states.states[all_states.current_state].numinputs&&!processed)
      {
        if(all_states.states[all_states.current_state].inputs[i].inpchar==GNPK_UPPERCASE)
        {
          if(gnp_global_data.program_debug)
            printf("PROGRAM: Matched input for %%uppercase. Executing those commands.\n");
          gnp_execute_input(&all_states.states[all_states.current_state].inputs[i]);
          processed=TRUE;
        }
        i++;
      }
    }
  }

  if(!processed)
  {
    if(isalpha(keyval))
    {
      i=0;
      while(i<all_states.states[all_states.current_state].numinputs&&!processed)
      {
        if(all_states.states[all_states.current_state].inputs[i].inpchar==GNPK_ALPHA)
        {
          if(gnp_global_data.program_debug)
            printf("PROGRAM: Matched input for %%alpha. Executing those commands.\n");
          gnp_execute_input(&all_states.states[all_states.current_state].inputs[i]);
          processed=TRUE;
        }
        i++;
      }
    }
  }

  if(!processed)
  {
    if(isalnum(keyval))
    {
      i=0;
      while(i<all_states.states[all_states.current_state].numinputs&&!processed)
      {
        if(all_states.states[all_states.current_state].inputs[i].inpchar==GNPK_ALPHNUM)
        {
          if(gnp_global_data.program_debug)
            printf("PROGRAM: Matched input for %%alphnum. Executing those commands.\n");
          gnp_execute_input(&all_states.states[all_states.current_state].inputs[i]);
          processed=TRUE;
        }
        i++;
      }
    }
  }

  if(!processed)
  {
    if(ispunct(keyval))
    {
      i=0;
      while(i<all_states.states[all_states.current_state].numinputs&&!processed)
      {
        if(all_states.states[all_states.current_state].inputs[i].inpchar==GNPK_PUNCT)
        {
          if(gnp_global_data.program_debug)
            printf("PROGRAM: Matched input for %%punct. Executing those commands.\n");
          gnp_execute_input(&all_states.states[all_states.current_state].inputs[i]);
          processed=TRUE;
        }
        i++;
      }
    }
  }

  if(!processed)
  {
    if(isspace(keyval))
    {
      i=0;
      while(i<all_states.states[all_states.current_state].numinputs&&!processed)
      {
        if(all_states.states[all_states.current_state].inputs[i].inpchar==GNPK_SPACES)
        {
          if(gnp_global_data.program_debug)
            printf("PROGRAM: Matched input for %%spaces. Executing those commands.\n");
          gnp_execute_input(&all_states.states[all_states.current_state].inputs[i]);
          processed=TRUE;
        }
        i++;
      }
    }
  }

  if(!processed)
  {
    if(isalnum(keyval)||isspace(keyval)||ispunct(keyval))
    {
      i=0;
      while(i<all_states.states[all_states.current_state].numinputs&&!processed)
      {
        if(all_states.states[all_states.current_state].inputs[i].inpchar==GNPK_PRINTING)
        {
          if(gnp_global_data.program_debug)
            printf("PROGRAM: Matched input for %%printing. Executing those commands.\n");
          gnp_execute_input(&all_states.states[all_states.current_state].inputs[i]);
          processed=TRUE;
        }
        i++;
      }
    }
  }

  if(!processed)
  {
    if(iscntrl(keyval))
    {
      i=0;
      while(i<all_states.states[all_states.current_state].numinputs&&!processed)
      {
        if(all_states.states[all_states.current_state].inputs[i].inpchar==GNPK_CONTROL)
        {
          if(gnp_global_data.program_debug)
            printf("PROGRAM: Matched input for %%control. Executing those commands.\n");
          gnp_execute_input(&all_states.states[all_states.current_state].inputs[i]);
          processed=TRUE;
        }
        i++;
      }
    }
  }

  if(!processed)
  {
    i=0;
    while(i<all_states.states[all_states.current_state].numinputs&&!processed)
    {
      if(all_states.states[all_states.current_state].inputs[i].inpchar==GNPK_DEFAULT)
      {
        if(gnp_global_data.program_debug)
          printf("PROGRAM: Matched input for %%default. Executing those commands.\n");
        gnp_execute_input(&all_states.states[all_states.current_state].inputs[i]);
        processed=TRUE;
      }
      i++;
    }
  }
}


void gnp_execute_input(state_input_struct *stateinput)

/* Call gnp_perform_command on the commands, Set  */
/* the end-state to the endstate for this state   */
/* and input combination.                         */

{
  commands_struct *do_me;

  do_me=stateinput->command;
  while(do_me!=NULL)
  {
    gnp_perform_command(do_me);
    do_me=do_me->next;
  }

  gnp_change_state(stateinput->endstate);
}


void gnp_perform_command(commands_struct *command)

/*  Lookup function in functions table, perform   */
/*  as necessary, set return value and leave.     */

{
  guint i=0;
  guint funcindex;

  if(gnp_global_data.program_debug)
  {
    printf("PROGRAM: Executing function code %x with",command->function);
    if(command->param_count>0)
      printf(" %d parameters\n",command->param_count);
    else
      printf(" no parameters\n");
  }

  funcindex=gnp_lookup_function(command->function);
  if(funcindex==0xffffffff)
  {
    printf("ERROR: Undefined function code to process. ABORTING\n");
    exit(-1);
  }
  if((function_table[funcindex].command==NULL)&&(function_table[funcindex].sub!=NULL)&&(command->function>0xfeffffff))
  {
    printf("test1\n");
    if(function_table[funcindex].param_count==0)
      function_table[funcindex].sub();
    else
      if(function_table[funcindex].param_count==1)
      {
        if(gnp_global_data.program_debug)
          printf("PROGRAM: param1- %x (%s)\n",command->param1,gnp_lookup_function_name(command->param1));
        function_table[funcindex].sub(command->param1);
      }

  }

}


void gnp_set_register_value(guint reg, guint value, guint type)

/*  Set the named register to the value specified  */
/* *should* work with both ints and pointers!      */
/* Silently fails if:                              */
/* - specified register is read-only (type is      */
/*    GNP_REGISTER_SPECIAL); or                    */
/* - specified register does not exist.            */
/* Since the latter should *never* occur, it       */
/* shouldn't matter.                               */

{
  gboolean nofind=TRUE;
  guint i;

  i=0;
  while(nofind&&i<register_table_count)
  {
    if(register_table[i].code==reg)
    {
      if(register_table[i].type!=GNP_REGISTER_SPECIAL)
      {
        register_table[i].value=value;
        register_table[i].value_type=type;
        nofind=FALSE;
      }
    }
    else
      i++;
  }
}

void gnp_set_register_value_privileged(guint reg, guint value, guint type)

/*  Set the named register to the value specified  */
/* *should* work with both ints and pointers!      */
/* The privileged version will set the value even  */
/* if the specified register is read_only (type == */
/*   GNP_REGISTER_SPECIAL).                        */
/*                                                 */
/*  Should ONLY be use to set the KEY, RETURN and  */
/*  STATENAME registers in appropriate places. NOT */
/*  FOR USE IN STATE PROGRAM CALLABLE ROUTINES!    */
/*  (with the obvious exception of the "return"    */
/*  function which does need to set the RETURN     */
/*  register).                                     */

{
  gboolean nofind=TRUE;
  guint i;

  i=0;
  while(nofind&&i<register_table_count)
  {
    if(register_table[i].code==reg)
    {
      register_table[i].value=value;
      register_table[i].value_type=type;
      nofind=FALSE;
    }
    else
      i++;
  }
}

guint gnp_get_register_value(guint reg)

/*  Returns the value of the register   */

{
  guint i;

  i=0;
  while(i<register_table_count)
  {
    if(register_table[i].code==reg)
      return register_table[i].value;
    else
      i++;
  }
  return 0;
}  

guint gnp_get_register_type(guint reg)

/*  Returns the type of the register value */

{
  guint i;

  i=0;
  while(i<register_table_count)
  {
    if(register_table[i].code==reg)
      return register_table[i].value_type;
    else
      i++;
  }
  return GNP_VALUE_UNDEF;
}  

void gnp_change_state(guint newstate)

/*  Do all processing necessary to change  */
/*  to the new state.                      */

{
  /* If not state change required, just return */

/*  if(newstate==all_states.current_state)
    return; */

  /* Display new helptext if it exists  */

  if(all_states.states[newstate].helptext!=NULL)
  {
    gnp_rightbox_erase_all();
    gnp_rightbox_insert_string(all_states.states[newstate].helptext);
  }

  all_states.current_state=newstate;
}

void gnp_reset_func()

/* Processing for "reset" command  */

{
  printf("IN RESET COMMAND PROCESSING\n");
}

void gnp_clear_func(guint param1)

/* Processing for "clear" command  */

{
  printf("IN CLEAR COMMAND PROCESSING\n");
  printf("param1 would be: %x\n",param1);
}